@@ -4,7 +4,6 @@ Python2 and Python3 based virtual environments. Environments are
 setup by using virtualenv. Configuration is generally done through an
 INI-style "tox.ini" file.
 """
-from __future__ import print_function
 
 import os
 import pipes
@@ -13,14 +12,21 @@ import shutil
 import subprocess
 import sys
 import time
+from collections import OrderedDict
+from contextlib import contextmanager
+from threading import Event, Semaphore, Thread
 
 import pkg_resources
 import py
 
 import tox
 from tox.config import parseconfig
+from tox.config.parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
+from tox.config.parallel import OFF_VALUE as PARALLEL_OFF
 from tox.result import ResultLog
 from tox.util import set_os_env_var
+from tox.util.graph import stable_topological_sort
+from tox.util.spinner import Spinner
 from tox.venv import VirtualEnv
 
 
@@ -51,7 +57,7 @@ def main(args):
         raise SystemExit(retcode)
     except KeyboardInterrupt:
         raise SystemExit(2)
-    except tox.exception.MinVersionError as e:
+    except (tox.exception.MinVersionError, tox.exception.MissingRequirement) as e:
         r = Reporter(None)
         r.error(str(e))
         raise SystemExit(1)
@@ -67,10 +73,12 @@ def show_help(config):
     tw.line()
     tw.line("Environment variables", bold=True)
     tw.line("TOXENV: comma separated list of environments (overridable by '-e')")
+    tw.line("TOX_SKIP_ENV: regular expression to filter down from running tox environments")
     tw.line(
         "TOX_TESTENV_PASSENV: space-separated list of extra environment variables to be "
         "passed into test command environments"
     )
+    tw.line("PY_COLORS: 0 disable colorized output, 1 enable (default)")
 
 
 def show_help_ini(config):
@@ -140,10 +148,21 @@ class Action(object):
         f.flush()
         return f
 
-    def popen(self, args, cwd=None, env=None, redirect=True, returnout=False, ignore_ret=False):
+    def popen(
+        self,
+        args,
+        cwd=None,
+        env=None,
+        redirect=True,
+        returnout=False,
+        ignore_ret=False,
+        capture_err=True,
+    ):
         stdout = outpath = None
         resultjson = self.session.config.option.resultjson
 
+        stderr = subprocess.STDOUT if capture_err else None
+
         cmd_args = [str(x) for x in args]
         cmd_args_shell = " ".join(pipes.quote(i) for i in cmd_args)
         if resultjson or redirect:
@@ -164,7 +183,7 @@ class Action(object):
             # FIXME XXX cwd = self.session.config.cwd
             cwd = py.path.local()
         try:
-            popen = self._popen(args, cwd, env=env, stdout=stdout, stderr=subprocess.STDOUT)
+            popen = self._popen(args, cwd, env=env, stdout=stdout, stderr=stderr)
         except OSError as e:
             self.report.error(
                 "invocation failed (errno {:d}), args: {}, cwd: {}".format(
@@ -274,8 +293,6 @@ class Verbosity(object):
 
 
 class Reporter(object):
-    actionchar = "-"
-
     def __init__(self, session):
         self.tw = py.io.TerminalWriter()
         self.session = session
@@ -315,13 +332,14 @@ class Reporter(object):
         if self.verbosity >= Verbosity.QUIET:
             self.tw.sep("_", "summary")
 
-    def info(self, msg):
-        if self.verbosity >= Verbosity.DEBUG:
-            self.logline(msg)
+    def logline_if(self, level, msg, key=None, **kwargs):
+        if self.verbosity >= level:
+            message = str(msg) if key is None else "{}{}".format(key, msg)
+            self.logline(message, **kwargs)
 
-    def using(self, msg):
-        if self.verbosity >= Verbosity.INFO:
-            self.logline("using {}".format(msg), bold=True)
+    def logline(self, msg, **opts):
+        self.reported_lines.append(msg)
+        self.tw.line("{}".format(msg), **opts)
 
     def keyboard_interrupt(self):
         self.error("KEYBOARDINTERRUPT")
@@ -336,40 +354,32 @@ class Reporter(object):
     def line(self, msg, **opts):
         self.logline(msg, **opts)
 
+    def info(self, msg):
+        self.logline_if(Verbosity.DEBUG, msg)
+
+    def using(self, msg):
+        self.logline_if(Verbosity.INFO, msg, "using ", bold=True)
+
     def good(self, msg):
-        if self.verbosity >= Verbosity.QUIET:
-            self.logline(msg, green=True)
+        self.logline_if(Verbosity.QUIET, msg, green=True)
 
     def warning(self, msg):
-        if self.verbosity >= Verbosity.QUIET:
-            self.logline("WARNING: {}".format(msg), red=True)
+        self.logline_if(Verbosity.QUIET, msg, "WARNING: ", red=True)
 
     def error(self, msg):
-        if self.verbosity >= Verbosity.QUIET:
-            self.logline("ERROR: {}".format(msg), red=True)
+        self.logline_if(Verbosity.QUIET, msg, "ERROR: ", red=True)
 
     def skip(self, msg):
-        if self.verbosity >= Verbosity.QUIET:
-            self.logline("SKIPPED: {}".format(msg), yellow=True)
-
-    def logline(self, msg, **opts):
-        self.reported_lines.append(msg)
-        self.tw.line("{}".format(msg), **opts)
+        self.logline_if(Verbosity.QUIET, msg, "SKIPPED: ", yellow=True)
 
     def verbosity0(self, msg, **opts):
-        if self.verbosity >= Verbosity.DEFAULT:
-            self.logline("{}".format(msg), **opts)
+        self.logline_if(Verbosity.DEFAULT, msg, **opts)
 
     def verbosity1(self, msg, **opts):
-        if self.verbosity >= Verbosity.INFO:
-            self.logline("{}".format(msg), **opts)
+        self.logline_if(Verbosity.INFO, msg, **opts)
 
     def verbosity2(self, msg, **opts):
-        if self.verbosity >= Verbosity.DEBUG:
-            self.logline("{}".format(msg), **opts)
-
-    # def log(self, msg):
-    #    print(msg, file=sys.stderr)
+        self.logline_if(Verbosity.DEBUG, msg, **opts)
 
 
 class Session:
@@ -389,8 +399,15 @@ class Session:
             self.venvlist = [self.getvenv(x) for x in self.evaluated_env_list()]
         except LookupError:
             raise SystemExit(1)
-        except tox.exception.ConfigError as e:
-            self.report.error(str(e))
+        except tox.exception.ConfigError as exception:
+            self.report.error(str(exception))
+            raise SystemExit(1)
+        try:
+            self.venv_order = stable_topological_sort(
+                OrderedDict((v.name, v.envconfig.depends) for v in self.venvlist)
+            )
+        except ValueError as exception:
+            self.report.error("circular dependency detected: {}".format(exception))
             raise SystemExit(1)
         self._actions = []
 
@@ -446,17 +463,25 @@ class Session:
         elif self.config.option.listenvs_all:
             self.showenvs(all_envs=True, description=verbosity)
         else:
-            return self.subcommand_test()
+            with self.cleanup():
+                return self.subcommand_test()
 
-    def _copyfiles(self, srcdir, pathlist, destdir):
-        for relpath in pathlist:
-            src = srcdir.join(relpath)
-            if not src.check():
-                self.report.error("missing source file: {}".format(src))
-                raise SystemExit(1)
-            target = destdir.join(relpath)
-            target.dirpath().ensure(dir=1)
-            src.copy(target)
+    @contextmanager
+    def cleanup(self):
+        self.config.temp_dir.ensure(dir=True)
+        try:
+            yield
+        finally:
+            for name in self.venv_order:
+                tox_env = self.getvenv(name)
+                if (
+                    hasattr(tox_env, "package")
+                    and isinstance(tox_env.package, py.path.local)
+                    and tox_env.package.exists()
+                ):
+                    self.report.verbosity2("cleanup {}".format(tox_env.package))
+                    tox_env.package.remove()
+                    py.path.local(tox_env.package.dirname).remove(ignore_errors=True)
 
     def make_emptydir(self, path):
         if path.check():
@@ -548,14 +573,27 @@ class Session:
         if self.config.skipsdist:
             self.report.info("skipping sdist step")
         else:
-            for venv in self.venvlist:
+            for name in self.venv_order:
+                venv = self.getvenv(name)
                 if not venv.envconfig.skip_install:
                     venv.package = self.hook.tox_package(session=self, venv=venv)
                     if not venv.package:
                         return 2
+                    venv.envconfig.setenv[str("TOX_PACKAGE")] = str(venv.package)
         if self.config.option.sdistonly:
             return
-        for venv in self.venvlist:
+
+        within_parallel = PARALLEL_ENV_VAR_KEY in os.environ
+        if not within_parallel and self.config.option.parallel != PARALLEL_OFF:
+            self.run_parallel()
+        else:
+            self.run_sequential()
+        retcode = self._summary()
+        return retcode
+
+    def run_sequential(self):
+        for name in self.venv_order:
+            venv = self.getvenv(name)
             if self.setupenv(venv):
                 if venv.envconfig.skip_install:
                     self.finishvenv(venv)
@@ -568,9 +606,105 @@ class Session:
                         self.installpkg(venv, venv.package)
 
                 self.runenvreport(venv)
-                self.runtestenv(venv)
-        retcode = self._summary()
-        return retcode
+            self.runtestenv(venv)
+
+    def run_parallel(self):
+        """here we'll just start parallel sub-processes"""
+        live_out = self.config.option.parallel_live
+        args = [sys.executable, "-m", "tox"] + self.config.args
+        try:
+            position = args.index("--")
+        except ValueError:
+            position = len(args)
+        try:
+            parallel_at = args[0:position].index("--parallel")
+            del args[parallel_at]
+            position -= 1
+        except ValueError:
+            pass
+
+        max_parallel = self.config.option.parallel
+        if max_parallel is None:
+            max_parallel = len(self.venv_order)
+        semaphore = Semaphore(max_parallel)
+        finished = Event()
+        sink = None if live_out else subprocess.PIPE
+
+        show_progress = not live_out and self.report.verbosity > Verbosity.QUIET
+        with Spinner(enabled=show_progress) as spinner:
+
+            def run_in_thread(tox_env, os_env):
+                res = None
+                env_name = tox_env.envconfig.envname
+                try:
+                    os_env[str(PARALLEL_ENV_VAR_KEY)] = str(env_name)
+                    args_sub = list(args)
+                    if hasattr(tox_env, "package"):
+                        args_sub.insert(position, str(tox_env.package))
+                        args_sub.insert(position, "--installpkg")
+                    process = subprocess.Popen(
+                        args_sub,
+                        env=os_env,
+                        stdout=sink,
+                        stderr=sink,
+                        stdin=None,
+                        universal_newlines=True,
+                    )
+                    res = process.wait()
+                finally:
+                    semaphore.release()
+                    finished.set()
+                    tox_env.status = (
+                        "skipped tests"
+                        if self.config.option.notest
+                        else ("parallel child exit code {}".format(res) if res else res)
+                    )
+                    done.add(env_name)
+                    report = spinner.succeed
+                    if self.config.option.notest:
+                        report = spinner.skip
+                    elif res:
+                        report = spinner.fail
+                    report(env_name)
+
+                if not live_out:
+                    out, err = process.communicate()
+                    if res or tox_env.envconfig.parallel_show_output:
+                        outcome = (
+                            "Failed {} under process {}, stdout:\n".format(env_name, process.pid)
+                            if res
+                            else ""
+                        )
+                        message = "{}{}{}".format(
+                            outcome, out, "\nstderr:\n{}".format(err) if err else ""
+                        ).rstrip()
+                        self.report.logline_if(Verbosity.QUIET, message)
+
+            threads = []
+            todo_keys = set(self.venv_order)
+            todo = OrderedDict(
+                (i, todo_keys & set(self.getvenv(i).envconfig.depends)) for i in self.venv_order
+            )
+            done = set()
+            while todo:
+                for name, depends in list(todo.items()):
+                    if depends - done:
+                        # skip if has unfinished dependencies
+                        continue
+                    del todo[name]
+                    venv = self.getvenv(name)
+                    semaphore.acquire(blocking=True)
+                    spinner.add(name)
+                    thread = Thread(target=run_in_thread, args=(venv, os.environ.copy()))
+                    thread.start()
+                    threads.append(thread)
+                if todo:
+                    # wait until someone finishes and retry queuing jobs
+                    finished.wait()
+                    finished.clear()
+
+            for thread in threads:
+                thread.join()
 
     def runenvreport(self, venv):
         """
@@ -584,51 +718,56 @@ class Session:
         envlog.set_installed(packages)
 
     def runtestenv(self, venv, redirect=False):
-        if not self.config.option.notest:
+        if venv.status == 0 and self.config.option.notest:
+            venv.status = "skipped tests"
+        else:
             if venv.status:
                 return
             self.hook.tox_runtest_pre(venv=venv)
             if venv.status == 0:
                 self.hook.tox_runtest(venv=venv, redirect=redirect)
             self.hook.tox_runtest_post(venv=venv)
-        else:
-            venv.status = "skipped tests"
 
     def _summary(self):
-        self.report.startsummary()
-        retcode = 0
-        for venv in self.venvlist:
+        is_parallel_child = PARALLEL_ENV_VAR_KEY in os.environ
+        if not is_parallel_child:
+            self.report.startsummary()
+        exit_code = 0
+        for name in self.venv_order:
+            venv = self.getvenv(name)
+            reporter = self.report.good
             status = venv.status
             if isinstance(status, tox.exception.InterpreterNotFound):
                 msg = " {}: {}".format(venv.envconfig.envname, str(status))
                 if self.config.option.skip_missing_interpreters == "true":
-                    self.report.skip(msg)
+                    reporter = self.report.skip
                 else:
-                    retcode = 1
-                    self.report.error(msg)
+                    exit_code = 1
+                    reporter = self.report.error
             elif status == "platform mismatch":
                 msg = " {}: {}".format(venv.envconfig.envname, str(status))
-                self.report.skip(msg)
+                reporter = self.report.skip
             elif status and status == "ignored failed command":
                 msg = "  {}: {}".format(venv.envconfig.envname, str(status))
-                self.report.good(msg)
             elif status and status != "skipped tests":
                 msg = "  {}: {}".format(venv.envconfig.envname, str(status))
-                self.report.error(msg)
-                retcode = 1
+                reporter = self.report.error
+                exit_code = 1
             else:
                 if not status:
                     status = "commands succeeded"
-                self.report.good("  {}: {}".format(venv.envconfig.envname, status))
-        if not retcode:
+                msg = "  {}: {}".format(venv.envconfig.envname, status)
+            if not is_parallel_child:
+                reporter(msg)
+        if not exit_code and not is_parallel_child:
             self.report.good("  congratulations :)")
-
-        path = self.config.option.resultjson
-        if path:
-            path = py.path.local(path)
-            path.write(self.resultlog.dumps_json())
-            self.report.line("wrote json report at: {}".format(path))
-        return retcode
+        if not is_parallel_child:
+            path = self.config.option.resultjson
+            if path:
+                path = py.path.local(path)
+                path.write(self.resultlog.dumps_json())
+                self.report.line("wrote json report at: {}".format(path))
+        return exit_code
 
     def showconfig(self):
         self.info_versions()
