@@ -4,10 +4,10 @@ import os
 import pipes
 import re
 import sys
-import warnings
 from itertools import chain
 
 import py
+from pkg_resources import to_filename
 
 import tox
 
@@ -174,7 +174,8 @@ class VirtualEnv(object):
                 "  cmd: {}\n"
                 "  env: {}\n"
                 "Maybe you forgot to specify a dependency? "
-                "See also the whitelist_externals envconfig setting.".format(
+                "See also the whitelist_externals envconfig setting.\n\n"
+                "DEPRECATION WARNING: this will be an error in tox 4 and above!".format(
                     path, self.envconfig.envdir
                 )
             )
@@ -243,14 +244,6 @@ class VirtualEnv(object):
             alwayscopy,
         )
 
-    def _getresolvedeps(self):
-        warnings.warn(
-            "that's a private function there, use get_resolved_dependencies,"
-            "this will be removed in 3.2",
-            category=DeprecationWarning,
-        )
-        return self.get_resolved_dependencies()
-
     def get_resolved_dependencies(self):
         dependencies = []
         for dependency in self.envconfig.deps:
@@ -281,7 +274,9 @@ class VirtualEnv(object):
         setup_cfg = setupdir.join("setup.cfg")
         args = [self.envconfig.envpython, str(setup_py), "--name"]
         env = self._get_os_environ()
-        output = action.popen(args, cwd=setupdir, redirect=False, returnout=True, env=env)
+        output = action.popen(
+            args, cwd=setupdir, redirect=False, returnout=True, env=env, capture_err=False
+        )
         name = next(
             (i for i in output.split("\n") if i and not i.startswith("pydev debugger:")), ""
         )
@@ -295,7 +290,7 @@ class VirtualEnv(object):
             sys_path = json.loads(out)
         except ValueError:
             sys_path = []
-        egg_info_fname = ".".join((name, "egg-info"))
+        egg_info_fname = ".".join((to_filename(name), "egg-info"))
         for d in reversed(sys_path):
             egg_info = py.path.local(d).join(egg_info_fname)
             if egg_info.check():
@@ -564,6 +559,9 @@ def prepend_shebang_interpreter(args):
     return args
 
 
+NO_DOWNLOAD = False
+
+
 @tox.hookimpl
 def tox_testenv_create(venv, action):
     config_interpreter = venv.getsupportedinterpreter()
@@ -572,6 +570,8 @@ def tox_testenv_create(venv, action):
         args.append("--system-site-packages")
     if venv.envconfig.alwayscopy:
         args.append("--always-copy")
+    if NO_DOWNLOAD:
+        args.append("--no-download")
     # add interpreter explicitly, to prevent using default (virtualenv.ini)
     args.extend(["--python", str(config_interpreter)])
     venv.session.make_emptydir(venv.path)
